"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __asyncValues = (this && this.__asyncValues) || function (o) {
    if (!Symbol.asyncIterator) throw new TypeError("Symbol.asyncIterator is not defined.");
    var m = o[Symbol.asyncIterator], i;
    return m ? m.call(o) : (o = typeof __values === "function" ? __values(o) : o[Symbol.iterator](), i = {}, verb("next"), verb("throw"), verb("return"), i[Symbol.asyncIterator] = function () { return this; }, i);
    function verb(n) { i[n] = o[n] && function (v) { return new Promise(function (resolve, reject) { v = o[n](v), settle(resolve, reject, v.done, v.value); }); }; }
    function settle(resolve, reject, d, v) { Promise.resolve(v).then(function(v) { resolve({ value: v, done: d }); }, reject); }
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.AuthNotificationService = void 0;
const local_environment_1 = require("../enviroments/local_environment");
const request_info_query_1 = require("../graphqlQueries/request-info.query");
const lodash_1 = __importDefault(require("lodash"));
const index_1 = require("../lib/lambda-logger/index");
const nitinolService_1 = require("./nitinolService");
class AuthNotificationService {
    constructor() {
        this.log = index_1.LambdaLogger.getInstance("AuthNotificationService");
        this.staus = ["new", "inprogress", "pending"];
    }
    sendSecretManagementNotification(appName, remainingDays, resetSecret, resetSecretStatusCode, isManualReset) {
        return __awaiter(this, void 0, void 0, function* () {
            let autoSecretRotation;
            const appInfo = yield this.getDataFromQuery(request_info_query_1.ALL_DEVELOPERS_APP, {
                appName: appName,
            });
            const tenantId = yield this.getTenantIdByAppName(appName);
            const tenantAttributes = yield this.getTenantAttributesByTenantId(tenantId);
            const tenantTemplateName = tenantAttributes['tenant_template_name'];
            const tenantEmailFromAddress = tenantAttributes['tenant_email_fromaddress'];
            let appDevelopers = lodash_1.default.map(appInfo.app[0].groups[0].group.users, "user.user_name");
            if (lodash_1.default.isEmpty(appDevelopers)) {
                this.log.info("Unable to fetch Developers List of Email Addresses for an App from DB");
            }
            var paladinPortalUrl = "paladin-portal.corp.hpicloud.net/";
            if (process.env.stage != "prod") {
                var envStage = "dev";
                if (process.env.stage === "test") {
                    envStage = "stg";
                }
                paladinPortalUrl =
                    "https://paladin-portal-" + envStage + ".corp.hpicloud.net/";
            }
            paladinPortalUrl += "apps/" + appInfo.app[0].id;
            let auto = (!isManualReset) ? ' auto' : '';
            let action = (!isManualReset) ? ' No action required from your end.' : '';
            const templateData = {
                remainingDays: remainingDays,
                env: process.env.stage,
                appName: appInfo.app[0].display_name,
                paladinPortalUrl: paladinPortalUrl,
                tenantName: tenantTemplateName,
                tenantEmailFromAddress: tenantEmailFromAddress,
                auto: auto,
                action: action
            };
            if (appInfo.app[0].attributes && appInfo.app[0].attributes.length > 0) {
                autoSecretRotation = lodash_1.default.find(appInfo.app[0].attributes, { name: 'enable-auto-secret-rotation', value: 'true' });
            }
            if (autoSecretRotation) {
                if (resetSecret) {
                    if (resetSecretStatusCode === 200) {
                        try {
                            if (process.env.stage == "prod") {
                                const serviceAppDetails = yield this.getDataFromQuery(request_info_query_1.GET_APP_DETAILS_BY_NAME, {
                                    appName: appName,
                                });
                                if (serviceAppDetails.app && serviceAppDetails.app.length > 0) {
                                    let appId = serviceAppDetails.app[0].id;
                                    let attributes = serviceAppDetails.app[0].attributes;
                                    const appType = attributes.find(({ name }) => name === 'app-type').value;
                                    attributes = lodash_1.default.filter(attributes, function (p) {
                                        return p.name !== 'secret-created-date' && p.name !== 'secret-expiry-date';
                                    });
                                    if (appType === 'service') {
                                        let secretCreatedDate = [{ name: 'secret-created-date', value: Math.round(Date.now() / 1000) }];
                                        attributes = [...attributes, ...secretCreatedDate];
                                        let updated = yield this.updateAppAttributes(appId, attributes);
                                        if (updated) {
                                            this.log.info("Updated the app attributes Value");
                                        }
                                    }
                                }
                            }
                        }
                        catch (error) {
                            this.log.info("Failed to get app data");
                        }
                        var params = {
                            templateName: "secretExpiryAutoRotationSuccessEmail",
                            toAddress: appDevelopers,
                            subject: tenantTemplateName + " Auth - " +
                                lodash_1.default.upperCase(process.env.stage) +
                                " : Secret Expiry Information for " + appInfo.app[0].display_name + ".",
                            templateData: templateData,
                        };
                    }
                    else {
                        var params = {
                            templateName: "secretExpiryAutoRotationFailedEmail",
                            toAddress: appDevelopers,
                            subject: tenantTemplateName + " Auth - " +
                                lodash_1.default.upperCase(process.env.stage) +
                                " : Secret Expiry Information for " + appInfo.app[0].display_name + ".",
                            templateData: templateData,
                        };
                    }
                }
                else {
                    if (remainingDays && parseInt(remainingDays) === 15 || parseInt(remainingDays) === 8) {
                        var params = {
                            templateName: "secretExpiryInformationAutoEmail",
                            toAddress: appDevelopers,
                            subject: tenantTemplateName + " Auth - " +
                                lodash_1.default.upperCase(process.env.stage) +
                                " : Secret Expiry Information for " + appInfo.app[0].display_name + ".",
                            templateData: templateData,
                        };
                    }
                    else {
                        return { status: 'Auto-Rotation info email should be sent on 15th and 8th day' };
                    }
                }
            }
            else {
                if (resetSecret) {
                    if (resetSecretStatusCode === 200) {
                        var params = {
                            templateName: "secretExpiryAutoRotationSuccessEmail",
                            toAddress: appDevelopers,
                            subject: tenantTemplateName + " Auth - " +
                                lodash_1.default.upperCase(process.env.stage) +
                                " : Secret Expiry Information for " + appInfo.app[0].display_name + ".",
                            templateData: templateData,
                        };
                    }
                    else {
                        var params = {
                            templateName: "secretExpiryAutoRotationFailedEmail",
                            toAddress: appDevelopers,
                            subject: tenantTemplateName + " Auth - " +
                                lodash_1.default.upperCase(process.env.stage) +
                                " : Secret Expiry Information for " + appInfo.app[0].display_name + ".",
                            templateData: templateData,
                        };
                    }
                }
                else {
                    var params = {
                        templateName: "secretExpiryWarningEmail",
                        toAddress: appDevelopers,
                        subject: tenantTemplateName + " Auth - " +
                            lodash_1.default.upperCase(process.env.stage) +
                            " : Secret Expiry Warning for " + appInfo.app[0].display_name + ".",
                        templateData: templateData,
                    };
                }
            }
            params["fromAddress"] = tenantEmailFromAddress;
            this.log.info("Sending email to: " + appDevelopers);
            var notificationStatus = yield this.sendEmail(params);
            return notificationStatus;
        });
    }
    sendNotification(requestMessage) {
        return __awaiter(this, void 0, void 0, function* () {
            const requestId = lodash_1.default.get(requestMessage, "event.data.new.id");
            const operation = lodash_1.default.get(requestMessage, "event.op");
            if (!requestId) {
                this.log.error("Request id is not present in the request message.");
                return;
            }
            const requestInfo = yield this.getDataFromQuery(request_info_query_1.REQUEST_INFO, {
                requestId,
            });
            const resourceId = lodash_1.default.get(requestInfo, "request[0].requested_data.resource");
            let admins;
            let approvers;
            let requestedApp;
            let sendUsingCustomEmailIdFlag = false;
            let customEmailId;
            if (!resourceId) {
                const paladinAdmin = yield this.getDataFromQuery(request_info_query_1.PALADIN_ADMINS, {});
                admins = lodash_1.default.map(paladinAdmin.group[0].users, "user.user_name");
                this.log.info("no resource for this request, hence sending the email to paladin admins", admins);
            }
            else {
                requestedApp = yield this.getDataFromQuery(request_info_query_1.REQUESTED_APP, {
                    resourceId,
                });
                const isCustomEmailId = lodash_1.default.find(requestedApp.app_product[0].app.attributes, { name: "notification-emailId" });
                if (isCustomEmailId) {
                    sendUsingCustomEmailIdFlag = true;
                    customEmailId = isCustomEmailId.value;
                }
                else {
                    sendUsingCustomEmailIdFlag = false;
                }
                this.log.info("Customize email : ", sendUsingCustomEmailIdFlag, " : ", customEmailId);
                const appId = lodash_1.default.get(requestedApp, "app_product[0].app.id");
                if (!appId) {
                    this.log.error("Unable to fetch App id from GraphQL Api");
                    return;
                }
                const allApprovers = yield this.getDataFromQuery(request_info_query_1.ALL_APPROVERS, {
                    appId: requestedApp.app_product[0].app.id,
                });
                approvers = lodash_1.default.map(allApprovers.app[0].groups[0].group.users, "user.user_name");
                if (lodash_1.default.isEmpty(approvers)) {
                    this.log.info("Unable to fetch Approvers List of Email Addresses for an App from DB");
                }
            }
            const params = yield this.getDataFromPayload(requestMessage, requestInfo, requestedApp, approvers, admins);
            if (!params) {
                this.log.info("No Data to send email");
                return;
            }
            if (sendUsingCustomEmailIdFlag) {
                params["fromAddress"] = customEmailId;
            }
            else if (params.templateData.tenantEmailFromAddress) {
                params["fromAddress"] = params.templateData.tenantEmailFromAddress;
            }
            else {
                params["fromAddress"] = null;
            }
            if (!lodash_1.default.isEmpty(params.toAddress)) {
                yield this.sendEmail(params);
            }
            if (params.templateName === "emailToAllApprovers") {
                if (operation === "INSERT") {
                    params.templateName = "emailToRequester";
                    params.subject =
                        params.templateData.tenantName + " Auth - " +
                            lodash_1.default.upperCase(process.env.stage) +
                            " : Your request is submitted successfully.";
                }
                if (operation === "UPDATE") {
                    params.templateName = "requestSentBackEmail";
                    params.subject =
                        params.templateData.tenantName + " Auth - " +
                            lodash_1.default.upperCase(process.env.stage) +
                            " : Your request is Sentback.";
                }
                params.toAddress = [requestInfo.request[0].rquestedBy.user_name];
                this.log.info("Sending email to requester");
                yield this.sendEmail(params);
            }
            if (params.templateName === "reactivateEmailToAllApprovers") {
                if (operation === "INSERT") {
                    params.templateName = "reactivationEmailToRequester";
                    params.subject =
                        params.templateData.tenantName + " Auth - " +
                            lodash_1.default.upperCase(process.env.stage) +
                            " : Your request is submitted successfully.";
                }
                if (operation === "UPDATE") {
                    params.templateName = "requestSentBackEmail";
                    params.subject =
                        params.templateData.tenantName + " Auth - " +
                            lodash_1.default.upperCase(process.env.stage) +
                            " : Your request is Sentback.";
                }
                params.toAddress = [requestInfo.request[0].rquestedBy.user_name];
                this.log.info("Sending email to requester");
                yield this.sendEmail(params);
            }
            if (params.templateName === "reactivationEmailAppLevelToApprover") {
                if (operation === "INSERT") {
                    params.templateName = "reactivationEmailAppLevelToRequester";
                    params.subject =
                        params.templateData.tenantName + " Auth - " +
                            lodash_1.default.upperCase(process.env.stage) +
                            " : Your app level reactivation request is submitted successfully.";
                }
                if (operation === "UPDATE") {
                    params.templateName = "requestSentBackEmail";
                    params.subject =
                        params.templateData.tenantName + " Auth - " +
                            lodash_1.default.upperCase(process.env.stage) +
                            " : Your app level reactivation request is Sentback.";
                }
                params.toAddress = [requestInfo.request[0].rquestedBy.user_name];
                this.log.info("Sending email to requester");
                yield this.sendEmail(params);
            }
            if (params.templateName === "emailToAllApproversAfterResubmit") {
                params.templateName = "emailToAllRequesterAfterResubmit";
                params.subject =
                    params.templateData.tenantName + " Auth - " +
                        lodash_1.default.upperCase(process.env.stage) +
                        " : Your request is re-submitted successfully.";
                params.toAddress = [requestInfo.request[0].rquestedBy.user_name];
                this.log.info("Sending email to requester");
                yield this.sendEmail(params);
            }
            if (params.templateName === "reactivateEmailToAllApproversAfterResubmit") {
                params.templateName = "reactivateEmailToAllRequesterAfterResubmit";
                params.subject =
                    params.templateData.tenantName + " Auth - " +
                        lodash_1.default.upperCase(process.env.stage) +
                        " : Your request is re-submitted successfully.";
                params.toAddress = [requestInfo.request[0].rquestedBy.user_name];
                this.log.info("Sending email to requester");
                yield this.sendEmail(params);
            }
            if (params.templateName === "reactivationEmailAppLevelToAllApproverAfterResubmit") {
                params.templateName = "reactivationEmailAppLevelToAllRequesterAfterResubmit";
                params.subject =
                    params.templateData.tenantName + " Auth - " +
                        lodash_1.default.upperCase(process.env.stage) +
                        " : Your app level reactivation request is re-submitted successfully.";
                params.toAddress = [requestInfo.request[0].rquestedBy.user_name];
                this.log.info("Sending email to requester");
                yield this.sendEmail(params);
            }
            if (params.templateName === "emailToAllApproversAfterAccept") {
                let approvedBy = requestInfo.request[0].approvedBy ? requestInfo.request[0].approvedBy.display_name : "";
                params.templateName = "emailToAllRequesterAfterAccept";
                params.subject =
                    params.templateData.tenantName + " Auth - " +
                        lodash_1.default.upperCase(process.env.stage) +
                        " : Your request is accepted by " + approvedBy;
                params.toAddress = [requestInfo.request[0].rquestedBy.user_name];
                this.log.info("Sending email to requester");
                yield this.sendEmail(params);
            }
        });
    }
    sendEmail(params) {
        return __awaiter(this, void 0, void 0, function* () {
            this.log.info("Calling nitinol service to send email with params", params);
            const notificationRes = yield nitinolService_1.NitinolService.sendEmail(params.templateName, params.toAddress, params.fromAddress, null, null, params.subject, params.templateData);
            return notificationRes;
        });
    }
    updateAppAttributes(appId, attributes) {
        return __awaiter(this, void 0, void 0, function* () {
            let result = {};
            try {
                result = yield local_environment_1.clientApiKey.mutate({
                    mutation: request_info_query_1.UPDATE_APP_ATTRIBUTES,
                    variables: {
                        appId: appId,
                        attributes: attributes
                    },
                });
                this.log.info("Updated the secret created date in app attributes table: ", result);
                return result;
            }
            catch (err) {
                this.log.error("Error while  updating app attributes data in DB", err);
                return;
            }
        });
    }
    getDataFromQuery(query, variables) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                this.log.info("getDataFromQuery tenantId", variables); //remove
                this.log.info("getDataFromQuery query", query); //remove
                const result = yield local_environment_1.clientApiKey.query({
                    query,
                    variables,
                });
                this.log.info("Received DB Call Result: ", result && result.data ? JSON.stringify(result.data) : "NULL");
                return result.data;
            }
            catch (err) {
                this.log.error("Error while querying data from DB", err);
                return;
            }
        });
    }
    getDataFromPayload(requestMessage, requestInfo, requestedApp, approversArr, admins) {
        return __awaiter(this, void 0, void 0, function* () {
            let params = {};
            let environmentRequestUrl;
            try {
                const requestStatus = lodash_1.default.get(requestMessage, "event.data.new.status");
                const tenantId = lodash_1.default.get(requestMessage, "event.data.new.tenant_id");
                const requestId = lodash_1.default.get(requestMessage, "event.data.new.id");
                const comments = requestInfo.request[0].comments;
                const latestComment = comments ? comments[comments.length - 1] : "";
                const subjectId = requestInfo.request[0].requested_data.subject;
                const requestCount = lodash_1.default.get(requestMessage, "event.data.new.notification_data.count");
                const requestCountOld = lodash_1.default.get(requestMessage, "event.data.old.notification_data.count");
                const subjectName = yield this.getDataFromQuery(request_info_query_1.SUBJECT_NAME, {
                    subjectId,
                });
                const tenantAttributes = yield this.getTenantAttributesByTenantId(tenantId);
                const tenantTemplateName = tenantAttributes['tenant_template_name'];
                const tenantEmailFromAddress = tenantAttributes['tenant_email_fromaddress'];
                const regExpForURLs = /((http|https):\/\/)?(ftp:\/\/)?(www\.)?(http:\/\/(.*?)\/)?[a-zA-Z0-9@:%._\\+~#?&//=]{2,256}\.[a-z]{2,6}\b([-a-zA-Z0-9@:%._\\+~#?&//=]*)/g;
                let customEmailId;
                var environmentUrl = "paladin-portal.corp.hpicloud.net/home";
                if (process.env.stage != "prod") {
                    var envStage = "dev";
                    if (process.env.stage === "test") {
                        envStage = "stg";
                    }
                    environmentUrl =
                        "https://paladin-portal-" + envStage + ".corp.hpicloud.net/home";
                }
                var callbackUrl = lodash_1.default.find(requestedApp === null || requestedApp === void 0 ? void 0 : requestedApp.app_product[0].app.attributes, { name: "callback-url" });
                if (callbackUrl) {
                    environmentRequestUrl = callbackUrl.value;
                }
                else {
                    environmentRequestUrl = '';
                }
                if (requestedApp && requestedApp.app_product && requestedApp.app_product.length > 0) {
                    customEmailId = lodash_1.default.find(requestedApp.app_product[0].app.attributes, { name: "notification-emailId" });
                }
                var dontReply = "";
                if (customEmailId) {
                    dontReply = "";
                }
                else {
                    dontReply = "Please do not reply to this email. We are unable to respond to inquiries sent to this address.";
                }
                const templateData = {
                    environmentUrl: environmentUrl,
                    environmentRequestUrl,
                    dontReply,
                    requestId,
                    username: requestInfo.request[0].rquestedBy.display_name,
                    type: lodash_1.default.upperCase(requestInfo.request[0].type),
                    status: lodash_1.default.upperCase(requestInfo.request[0].status),
                    app: requestedApp ? requestedApp.app_product[0].app.display_name : "",
                    subjectName: subjectName.user[0].display_name,
                    subjectEmailAdd: subjectName.user[0].user_name,
                    enviroment: requestedApp
                        ? lodash_1.default.upperCase(requestedApp.app_product[0].environment.name)
                        : lodash_1.default.upperCase(process.env.stage),
                    environmentReactivate: lodash_1.default.upperCase(process.env.stage),
                    requestedBy: requestInfo.request[0].rquestedBy.display_name,
                    whatAccess: (requestInfo.request[0].requested_data.whatAccess && this.isJSON(requestInfo.request[0].requested_data.whatAccess)) ? this.isJSON(requestInfo.request[0].requested_data.whatAccess) : requestInfo.request[0].requested_data.whatAccess.length > 100 ?
                        requestInfo.request[0].requested_data.whatAccess.replace(regExpForURLs, "").substring(0, 100) + '...' :
                        requestInfo.request[0].requested_data.whatAccess.replace(regExpForURLs, ""),
                    whyAccess: requestInfo.request[0].requested_data.whyAccess.length > 100 ?
                        requestInfo.request[0].requested_data.whyAccess.replace(regExpForURLs, "").substring(0, 100) + '...' :
                        requestInfo.request[0].requested_data.whyAccess.replace(regExpForURLs, ""),
                    email_id: requestInfo.request[0].rquestedBy.email_id,
                    comment: latestComment.note ? latestComment.note : "",
                    approverUsername: requestInfo.request[0].approvedBy
                        ? requestInfo.request[0].approvedBy.display_name
                        : "",
                    tenantName: tenantTemplateName,
                    tenantEmailFromAddress: tenantEmailFromAddress
                };
                /*
                  When request is new or pending or inprogress:
                  1. Cron job will run to send the reminder for auto reject
                */
                if (this.staus.includes(requestStatus) && requestCount && (requestCount != requestCountOld)) { //&& requestCount != 0
                    if (requestCount <= 3) {
                        params = {
                            templateName: "requestWaitingEmail",
                            toAddress: lodash_1.default.isEmpty(approversArr) ? admins : approversArr,
                            subject: "Reminder-" + requestCount + " - " + tenantTemplateName + " Auth - " +
                                lodash_1.default.upperCase(process.env.stage) +
                                " : The request is waiting for your action ",
                            templateData,
                        };
                    }
                    this.log.info("params", params);
                }
                /*
                  Request status is new :
                  1. New request raised
                  2. Request resubmitted after it is sentback by approver
                */
                if (requestStatus === "new" &&
                    requestInfo.request[0].type !== "reactivation" && (!requestCount || requestCount == requestCountOld) //|| requestCount == 0
                ) {
                    const oldStatus = lodash_1.default.get(requestMessage, "event.data.old.status");
                    // incase when request is sentback
                    if (oldStatus === "inprogress") {
                        params = {
                            templateName: "emailToAllApprovers",
                            templateData,
                        };
                        return params;
                    }
                    let subject = (oldStatus) ? ' : A request has been re-submitted and waiting for approval.' : ' : A new request has been submitted and waiting for approval.';
                    params = {
                        templateName: oldStatus
                            ? "emailToAllApproversAfterResubmit"
                            : "emailToAllApprovers",
                        toAddress: lodash_1.default.isEmpty(approversArr) ? admins : approversArr,
                        subject: tenantTemplateName + " Auth - " +
                            lodash_1.default.upperCase(process.env.stage) +
                            subject,
                        templateData,
                    };
                }
                // when request is accepted
                if (requestStatus === "inprogress" && (!requestCount || requestCount == requestCountOld)) {
                    params = {
                        templateName: "emailToAllApproversAfterAccept",
                        toAddress: lodash_1.default.isEmpty(approversArr) ? admins : approversArr,
                        subject: tenantTemplateName + " Auth - " +
                            lodash_1.default.upperCase(process.env.stage) +
                            " : The request is accepted by " + templateData.approverUsername,
                        templateData,
                    };
                    this.log.info("params", params);
                }
                if (requestInfo.request[0].type === "reactivation") {
                    if (requestInfo.request[0].requested_data.resource) {
                        //when user is disabled at app level, and raises an account reactivation request
                        templateData.type = templateData.type + " - App Level";
                        if (requestStatus === "new") {
                            const oldStatus = lodash_1.default.get(requestMessage, "event.data.old.status");
                            // incase when request is sentback
                            if (oldStatus === "inprogress") {
                                params = {
                                    templateName: "reactivationEmailAppLevelToApprover",
                                    templateData,
                                };
                                return params;
                            }
                            let subject = (oldStatus) ? ' : App Level reactivation request is re-submitted.' : ' : App Level reactivation request is raised.';
                            params = {
                                templateName: oldStatus
                                    ? "reactivationEmailAppLevelToAllApproverAfterResubmit"
                                    : "reactivationEmailAppLevelToApprover",
                                toAddress: lodash_1.default.isEmpty(approversArr) ? admins : approversArr,
                                subject: tenantTemplateName + " Auth - " +
                                    lodash_1.default.upperCase(process.env.stage) +
                                    subject,
                                templateData,
                            };
                        }
                        this.log.info("params", params);
                    }
                    else {
                        // when user is disabled, and raises an account reactivation request
                        if (requestStatus === "new") {
                            const oldStatus = lodash_1.default.get(requestMessage, "event.data.old.status");
                            // incase when request is sentback
                            if (oldStatus === "inprogress") {
                                params = {
                                    templateName: "reactivateEmailToAllApprovers",
                                    templateData,
                                };
                                return params;
                            }
                            let subject = (oldStatus) ? ' : Account reactivation request is re-submitted.' : ' : Account reactivation request is raised.';
                            params = {
                                templateName: oldStatus
                                    ? "reactivateEmailToAllApproversAfterResubmit"
                                    : "reactivateEmailToAllApprovers",
                                toAddress: lodash_1.default.isEmpty(approversArr) ? admins : approversArr,
                                subject: tenantTemplateName + " Auth - " +
                                    lodash_1.default.upperCase(process.env.stage) +
                                    subject,
                                templateData,
                            };
                        }
                        this.log.info("params", params);
                    }
                }
                // when request is reassigned
                if (requestStatus === "pending" && (!requestCount || requestCount == requestCountOld)) {
                    const approverId = lodash_1.default.get(requestMessage, "event.data.new.approved_by");
                    if (!approverId) {
                        this.log.error("Request hasn't been assigned to any approver, hence unable to get approver id");
                        return;
                    }
                    if (approverId) {
                        const approverEmailAdd = yield this.getDataFromQuery(request_info_query_1.APPROVER, {
                            approverId,
                        });
                        if (!approverEmailAdd) {
                            this.log.error("Unable to fetch Approver Email Address for an App from DB, hence not sending the email");
                            return;
                        }
                        params = {
                            templateName: "reassignRequestEmail",
                            toAddress: [approverEmailAdd.user[0].user_name],
                            subject: tenantTemplateName + " Auth - " +
                                lodash_1.default.upperCase(process.env.stage) +
                                " : A new request has been assigned to you, waiting for approval.",
                            templateData,
                        };
                    }
                }
                // when request is approved
                if (requestStatus === "approved") {
                    params = {
                        templateName: "requestApprovedEmail",
                        toAddress: [requestInfo.request[0].rquestedBy.user_name],
                        subject: tenantTemplateName + " Auth - " +
                            lodash_1.default.upperCase(process.env.stage) +
                            " : Your request is Approved successfully.",
                        templateData,
                    };
                }
                // when request is rejected
                if (requestStatus === "rejected") {
                    params = {
                        templateName: "requestRejectedEmail",
                        toAddress: [requestInfo.request[0].rquestedBy.user_name],
                        subject: tenantTemplateName + " Auth - " +
                            lodash_1.default.upperCase(process.env.stage) +
                            " : Your request is Rejected.",
                        templateData,
                    };
                    if (requestCount && requestCount == 4) {
                        params = {
                            templateName: "requestAutoRejectedEmail",
                            toAddress: [requestInfo.request[0].rquestedBy.user_name],
                            subject: tenantTemplateName + " Auth - " +
                                lodash_1.default.upperCase(process.env.stage) +
                                " : Your request is Auto Rejected.",
                            templateData,
                        };
                    }
                }
                return params;
            }
            catch (err) {
                this.log.error("Error while building email params:", err);
                return;
            }
        });
    }
    sendInactiveUserNotification(userEmail, emailContent, tenant_id) {
        return __awaiter(this, void 0, void 0, function* () {
            this.log.info("sendInactiveUserNotification getting called");
            this.log.info("userEmail" + userEmail + "emailContent" + emailContent + "tenant_id" + tenant_id);
            const tenantAttributes = yield this.getTenantAttributesByTenantId(tenant_id);
            this.log.info("tenantAttributes", tenantAttributes);
            const tenantTemplateName = tenantAttributes["tenant_template_name"];
            const tenantEmailFromAddress = tenantAttributes["tenant_email_fromaddress"];
            this.log.info("tenantTemplateName", tenantTemplateName);
            this.log.info("tenantEmailFromAddress", tenantEmailFromAddress);
            const templateData = {
                emailContent: emailContent,
                env: process.env.stage,
                dontReply: "Please do not reply to this email. We are unable to respond to inquiries sent to this address.",
                tenantName: tenantTemplateName,
            };
            this.log.info("templateData", templateData);
            userEmail = userEmail.split(" ");
            this.log.info("userEmail", userEmail);
            const params = {
                templateName: "paladinUserInactiveNotification",
                toAddress: userEmail,
                fromAddress: tenantEmailFromAddress,
                subject: tenantTemplateName +
                    " Auth - " +
                    lodash_1.default.upperCase(process.env.stage) +
                    " : Account Deactivation Reminder.",
                templateData: templateData,
            };
            this.log.info("params:", params);
            try {
                this.log.info("Sending email to: " + userEmail);
                const notificationStatus = yield this.sendEmail(params);
                return notificationStatus;
            }
            catch (err) {
                this.log.error("Sending email failed", err);
            }
        });
    }
    sendProductErrorNotification(usersEmailList, emailContent, tenantId) {
        return __awaiter(this, void 0, void 0, function* () {
            const tenantAttributes = yield this.getTenantAttributesByTenantId(tenantId);
            const tenantTemplateName = tenantAttributes['tenant_template_name'];
            const tenantEmailFromAddress = tenantAttributes['tenant_email_fromaddress'];
            const templateData = {
                emailContent: emailContent,
                env: process.env.stage,
                dontReply: "Please do not reply to this email. We are unable to respond to inquiries sent to this address.",
                tenantName: tenantTemplateName
            };
            var params = {
                templateName: "portalNotificationEmail",
                toAddress: usersEmailList,
                fromAddress: tenantEmailFromAddress,
                subject: tenantTemplateName + " Auth - " +
                    lodash_1.default.upperCase(process.env.stage) +
                    " : High Error Rate.",
                templateData: templateData,
            };
            try {
                if (usersEmailList.length > 45) {
                    var chunkSize = 45;
                    for (let i = 0; i <= usersEmailList.length; i += chunkSize) {
                        const chunkEmailIds = usersEmailList.slice(i, i + chunkSize);
                        params.toAddress = chunkEmailIds;
                        this.log.info("Sending batch email: " + chunkEmailIds);
                        var notificationStatus = yield this.sendEmail(params);
                    }
                }
                else {
                    this.log.info("Sending email to : " + usersEmailList);
                    var notificationStatus = yield this.sendEmail(params);
                }
                return notificationStatus;
            }
            catch (err) {
                this.log.error("Sending email failed", err);
            }
        });
    }
    sendPortalNotification(usersEmailList, emailContent, tenantId) {
        return __awaiter(this, void 0, void 0, function* () {
            const tenantAttributes = yield this.getTenantAttributesByTenantId(tenantId);
            const tenantTemplateName = tenantAttributes['tenant_template_name'];
            const tenantEmailFromAddress = tenantAttributes['tenant_email_fromaddress'];
            const templateData = {
                emailContent: emailContent,
                env: process.env.stage,
                dontReply: "Please do not reply to this email. We are unable to respond to inquiries sent to this address.",
                tenantName: tenantTemplateName
            };
            var params = {
                templateName: "portalNotificationEmail",
                toAddress: usersEmailList,
                fromAddress: tenantEmailFromAddress,
                subject: tenantTemplateName + " Auth - " +
                    lodash_1.default.upperCase(process.env.stage) +
                    " : Paladin Platform Notification.",
                templateData: templateData,
            };
            try {
                if (usersEmailList.length > 45) {
                    var chunkSize = 45;
                    for (let i = 0; i <= usersEmailList.length; i += chunkSize) {
                        const chunkEmailIds = usersEmailList.slice(i, i + chunkSize);
                        params.toAddress = chunkEmailIds;
                        this.log.info("Sending batch email: " + chunkEmailIds);
                        var notificationStatus = yield this.sendEmail(params);
                    }
                }
                else {
                    this.log.info("Sending email to : " + usersEmailList);
                    var notificationStatus = yield this.sendEmail(params);
                }
                return notificationStatus;
            }
            catch (err) {
                this.log.error("Sending email failed", err);
            }
        });
    }
    getTenantAttributesByTenantId(tenantId) {
        var _a, e_1, _b, _c;
        return __awaiter(this, void 0, void 0, function* () {
            try {
                this.log.info("tenantId", tenantId); //remove
                const tenantDetails = yield this.getDataFromQuery(request_info_query_1.GET_TENANT_DETAILS_BY_ID, {
                    tenantId
                });
                this.log.info("tenantDetails", tenantDetails); //remove
                let tenant_template_name;
                let tenantEmailFromAddress = null;
                if (tenantDetails.tenant && tenantDetails.tenant[0].attributes.length > 0) {
                    try {
                        for (var _d = true, _e = __asyncValues(tenantDetails.tenant[0].attributes), _f; _f = yield _e.next(), _a = _f.done, !_a;) {
                            _c = _f.value;
                            _d = false;
                            try {
                                const attribute = _c;
                                if (attribute.name == 'tenant-template-name') {
                                    tenant_template_name = attribute.value;
                                }
                                if (attribute.name == 'tenant-notification-fromaddress') {
                                    tenantEmailFromAddress = attribute.value;
                                }
                            }
                            finally {
                                _d = true;
                            }
                        }
                    }
                    catch (e_1_1) { e_1 = { error: e_1_1 }; }
                    finally {
                        try {
                            if (!_d && !_a && (_b = _e.return)) yield _b.call(_e);
                        }
                        finally { if (e_1) throw e_1.error; }
                    }
                }
                return { "tenant_template_name": tenant_template_name, "tenant_email_fromaddress": tenantEmailFromAddress };
            }
            catch (e) {
                this.log.error("failed to get tenant attributes", e);
            }
        });
    }
    getTenantIdByAppName(appName) {
        return __awaiter(this, void 0, void 0, function* () {
            const getappTenantId = yield this.getDataFromQuery(request_info_query_1.GET_APP_DETAILS_BY_NAME, {
                appName
            });
            if (getappTenantId.app && getappTenantId.app.length > 0) {
                return getappTenantId.app[0].tenant_id;
            }
            return;
        });
    }
    sendJiraNotification(data, jiraDetails, tenantId) {
        return __awaiter(this, void 0, void 0, function* () {
            const tenantAttributes = yield this.getTenantAttributesByTenantId(tenantId);
            const tenantTemplateName = tenantAttributes['tenant_template_name'];
            const tenantEmailFromAddress = tenantAttributes['tenant_email_fromaddress'];
            let emailContent = jiraDetails.fields.description;
            let reporter = jiraDetails.fields.reporter.name;
            let assignee = jiraDetails.fields.assignee.name;
            const usersEmailList = [reporter, assignee, "hp.mps.commonauth@groups.hp.com"];
            const templateData = {
                emailContent: emailContent,
                env: process.env.stage,
                dontReply: "Please do not reply to this email. We are unable to respond to inquiries sent to this address.",
                tenantName: tenantTemplateName
            };
            var params = {
                templateName: "paladinJiraNotification",
                toAddress: usersEmailList,
                fromAddress: tenantEmailFromAddress,
                subject: data.key,
                templateData: templateData,
            };
            this.log.info("params : " + params);
            try {
                this.log.info("Sending email to : " + usersEmailList);
                let notificationStatus = yield this.sendEmail(params);
                return notificationStatus;
            }
            catch (err) {
                this.log.error("Sending email failed", err);
            }
        });
    }
    isJSON(str) {
        try {
            return (JSON.parse(str).info && !!str) ? JSON.parse(str).info : "";
        }
        catch (e) {
            return false;
        }
    }
}
exports.AuthNotificationService = AuthNotificationService;
