"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __asyncValues = (this && this.__asyncValues) || function (o) {
    if (!Symbol.asyncIterator) throw new TypeError("Symbol.asyncIterator is not defined.");
    var m = o[Symbol.asyncIterator], i;
    return m ? m.call(o) : (o = typeof __values === "function" ? __values(o) : o[Symbol.iterator](), i = {}, verb("next"), verb("throw"), verb("return"), i[Symbol.asyncIterator] = function () { return this; }, i);
    function verb(n) { i[n] = o[n] && function (v) { return new Promise(function (resolve, reject) { v = o[n](v), settle(resolve, reject, v.done, v.value); }); }; }
    function settle(resolve, reject, d, v) { Promise.resolve(v).then(function(v) { resolve({ value: v, done: d }); }, reject); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.PublisherService = void 0;
const aws_sdk_1 = require("aws-sdk");
const axios = require("axios");
const index_1 = require("../lib/lambda-logger/index");
const local_environment_1 = require("../enviroments/local_environment");
const config_1 = require("../config");
const request_info_query_1 = require("../graphqlQueries/request-info.query");
class PublisherService {
    constructor() {
        this.log = index_1.LambdaLogger.getInstance("PublisherService");
    }
    publishResetKeyStatus(appName, autoRotationEnabled, clientId, status) {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            const tenantId = yield this.getTenantIdByAppName(appName);
            if (tenantId) {
                const tenantTemplateName = yield this.getTenantTemplateNameByTenantId(tenantId);
                if (tenantTemplateName) {
                    const params = {
                        Subject: tenantTemplateName + ' Auth - ' + ((_a = process.env.stage) === null || _a === void 0 ? void 0 : _a.toUpperCase()) + ' : Reset secret key status for ' + appName,
                        Message: 'Details: \n\n - App Name: ' + appName + '\n - Client ID: ' + clientId + '\n - Auto rotation enabled: ' + autoRotationEnabled + '\n - Status: ' + status,
                        TopicArn: (0, config_1.Configuration)().configServiceSNS
                    };
                    yield this.publishEventToMSTeams(params.Subject, params.Message);
                    yield this.publishEventToSNS(params);
                }
            }
            return;
        });
    }
    publishEventToSNS(params) {
        return __awaiter(this, void 0, void 0, function* () {
            const options = {
                endpoint: "https://sns.us-east-1.amazonaws.com",
                region: "us-east-1",
            };
            const snsClient = new aws_sdk_1.SNS(options);
            try {
                const result = yield snsClient.publish(params).promise();
                if (result) {
                    this.log.info("Successfully published event." + JSON.stringify(result));
                    return;
                }
                else {
                    this.log.error("Failed to publish event" + JSON.stringify(result));
                    return;
                }
            }
            catch (error) {
                this.log.error("Error while publishing event: " + error);
                return;
            }
        });
    }
    publishEventToMSTeams(title, text) {
        return __awaiter(this, void 0, void 0, function* () {
            const data = {
                "@context": "https://schema.org/extensions",
                "@type": "MessageCard",
                themeColor: "64a837",
                title,
                text
            };
            const config = {
                headers: {
                    "Content-Type": "application/json",
                },
            };
            try {
                const result = yield axios.post('' + (0, config_1.Configuration)().configServiceAlertsWebHook, data, config);
                if (result) {
                    this.log.info("Successfully published event to MSTeams");
                    return;
                }
                else {
                    this.log.error("Failed to publish event to MSTeams");
                    return;
                }
            }
            catch (error) {
                this.log.error("Error while publishing event to MSTeams: " + error);
                return;
            }
        });
    }
    getTenantTemplateNameByTenantId(tenantId) {
        var _a, e_1, _b, _c;
        return __awaiter(this, void 0, void 0, function* () {
            const tenantDetails = yield this.getDataFromQuery(request_info_query_1.GET_TENANT_DETAILS_BY_ID, {
                tenantId
            });
            if (tenantDetails.tenant && tenantDetails.tenant[0].attributes.length > 0) {
                try {
                    for (var _d = true, _e = __asyncValues(tenantDetails.tenant[0].attributes), _f; _f = yield _e.next(), _a = _f.done, !_a;) {
                        _c = _f.value;
                        _d = false;
                        try {
                            const attribute = _c;
                            if (attribute.name == 'tenant-template-name') {
                                return attribute.value;
                            }
                        }
                        finally {
                            _d = true;
                        }
                    }
                }
                catch (e_1_1) { e_1 = { error: e_1_1 }; }
                finally {
                    try {
                        if (!_d && !_a && (_b = _e.return)) yield _b.call(_e);
                    }
                    finally { if (e_1) throw e_1.error; }
                }
            }
            return "";
        });
    }
    getTenantIdByAppName(appName) {
        return __awaiter(this, void 0, void 0, function* () {
            const getappTenantId = yield this.getDataFromQuery(request_info_query_1.GET_APP_DETAILS_BY_NAME, {
                appName
            });
            if (getappTenantId.app && getappTenantId.app.length > 0) {
                return getappTenantId.app[0].tenant_id;
            }
            return;
        });
    }
    getDataFromQuery(query, variables) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const result = yield local_environment_1.clientApiKey.query({
                    query,
                    variables,
                });
                this.log.info("Received DB Call Result: ", result && result.data ? JSON.stringify(result.data) : "NULL");
                return result.data;
            }
            catch (err) {
                this.log.error("Error while querying data from DB", err);
                return;
            }
        });
    }
}
exports.PublisherService = PublisherService;
